/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#ifndef TEXTURE_H
#define TEXTURE_H

#include <iostream>
#include <string>
#include <cmath>
#include "Vector.h"

/**
 * class Texture
 * 
 */

class Texture
{
 public:
  Texture();
  Texture (const Texture& );
  Texture (const Vector& );
  ~Texture() {};
  virtual Vector getColor() const = 0;
  virtual Vector getColorPt(const Vector&) const = 0;
  virtual double getColor(int i) const = 0;

  double& operator[] (int i) { return color[i]; }

  const double operator[] (int i) const { return color[i]; }


 protected:
   Vector color;
 private:
};

class TextureDefault : public Texture 
{
 public:
  TextureDefault();
  TextureDefault(const Vector&);
  TextureDefault(const TextureDefault&);
  ~TextureDefault(){};

  virtual Vector getColor() const;
  virtual Vector getColorPt(const Vector&) const;
  virtual double getColor(int i) const;

};

//juste pour tester...
class TextureRandom : public Texture 
{
 public:
  TextureRandom(double min=0.0, double max=1.0);
  TextureRandom(const TextureRandom&);
  ~TextureRandom(){};

  virtual Vector getColor() const;
  virtual Vector getColorPt(const Vector&) const ;
  virtual double getColor(int i) const;
 private:
  double min;
  double max;
};

class TextureFct : public Texture 
{
 public:
  TextureFct(const Vector& , const Vector& );
  TextureFct(const TextureFct&);
  ~TextureFct(){};

  virtual Vector getColor() const;
  virtual Vector getColorPt(const Vector&) const ;
  virtual double getColor(int i) const;
 private:
  Vector color1, color2;
};

class TextureDamier : public Texture 
{
 public:
  TextureDamier(const Vector&, const Vector&, int step = 10);
  TextureDamier(const TextureDamier&);
  ~TextureDamier(){};

  virtual Vector getColor() const;
  virtual Vector getColorPt(const Vector&) const;
  virtual double getColor(int i) const;
 private:
  Vector color1, color2;
int step;

};

#endif // TEXTURE_H

