/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "Cone.h"

// Constructors/Destructors
//  


Cone::Cone (const Parameters& p, const Vector& a, const Vector& b , double r) : Node(p) {
  this->a = a;
  this->b = b;
  this->rayon = r;
  this->axe = Vector(1/(b[0]-a[0]),1/(b[1]-a[1]),1/(b[2]-a[2]));
  this->height = Norm(b-a);
  this->p.setPosition(a);
}

Cone::Cone (const Parameters& p, const Vector& a, const Vector& axe , double height, double r) : Node(p) {
  this->a = a;
  this->b = a + axe*height;
  this->rayon = r;
  this->axe = axe;
  this->height = height;

}


Cone::Cone (const Cone& c ) : Node(c.p){
  this->a = c.a;
  this->b = c.b;
  this->rayon = c.rayon;
  this->axe = c.axe;
  this->height = c.height;
}




int Cone::Inside (const Vector& p ) const{
  return 1;
}

int Cone::Inside (const Node&) const{
  return 1;
}

//x*x + y*y - z*z
int Cone::Intersect(const Ray& ray, Intersections& l) const{
  int result = 0;

  Vector dir=ray.Direction();
  Vector orig=a-ray.Origin();
  orig[1] = a[1] - ray.Origin()[1] + height;
  double e(-1.0*rayon*rayon/(height*height));

  double A = dir[0]*dir[0] + e*dir[1]*dir[1] + dir[2]*dir[2];
  double B = orig[0]*dir[0] + e*orig[1]*dir[1] + orig[2]*dir[2];
  double C = orig[0]*orig[0] + e*orig[1]*orig[1] + orig[2]*orig[2];
  double delta = B*B - A*C;

  if (delta > 0.0) {
    double d = sqrt(delta);
    double t2 = (B + d)/A;
    double t1 = (B - d)/A;
    Vector pt1 = ray(t1);
    Vector pt2 = ray(t2);
    if (t2 > 0.0){
      if(t1 < 0.0){

	if(!((pt1[1]>(a[1]+height))||(pt1[1]<a[1]))) {
	  Vector normal(0,-1,0);
	  // if (pt1[1]!=a[1]) {
	    normal[0] = pt1[0]-a[0];
	    normal[1] = (pt1[1]-a[1]-height)*e;
	    normal[2] = pt1[2]-a[2];
	    // }
	  l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(normal),pt1, t1));
	  result=1;
	}
      }

      if(!((pt2[1]>(a[1]+height))||(pt2[1]<a[1]))) {

	Vector normal(0,-1,0);
	//if (pt2[1]!=a[1]) {
	  normal[0] = pt2[0]-a[0];
	  normal[1] = (pt2[1]-a[1]-height)*e;
	  normal[2] = pt2[2]-a[2];
	  //}
	l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(normal),pt2, t2));
	result=1;
      }
    }
    if (result==0) {
      double t = dir[1]!=0 ? -( ray.Origin()[1] - a[1])/dir[1] : 0;
      if(t>0.0) {
	Vector pt =ray(t) - a;
	if(pt[0]*pt[0] + pt[2]*pt[2] <= rayon*rayon) {
	  Vector normal = (pt[1]==a[1]) ? Vector(0,-1,0) : Vector(pt[0]-a[0],(pt[1]-a[1]-height)*e, pt[2]-a[2]);	
	  l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(normal),pt, t));
	  result=1;
	}
      }
    }

  }

  return result;



}

