/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "Or.h"

// Constructors/Destructors
//  

Or::Or ( ) {}

Or::Or (const Node* n1, const Node* n2 ):Operator(n1,n2){}

Or::Or (const Or& o ):Operator(o){}

Or::~Or ( ) { delete n1; delete n2; }

//  
// Methods
//  


int Or::Inside (const Vector& p ) const{
  return (n1->Inside(p) || n2->Inside(p));
}

int Or::Inside (const Node&) const{
  return 1;
}

int Or::Intersect(const Ray& ray, Intersections& l) const{
  Intersections i1, i2;
  int result1 = n1->Intersect(ray,i1);
  int result2 = n2->Intersect(ray,i2);
   
  if(result1 && result2){
    if(i1.first().getDistance()<=i2.first().getDistance())
      l.append(i1.first());
    if(i1.first().getDistance()>=i2.first().getDistance())
      l.append(i2.first());
    return 1;
  }else if(result1){
    l.append(i1.first());
    return 1;
  }else if(result2){
    l.append(i2.first());
    return 1;
  }
  return 0;
}


// Accessor methods
//  


// Other methods
//  


