// Fast Raytracer
// Changelog 10.09.16


#include "Sphere.h"
#include <iostream>
#include <string>
/*!
  \class Sphere sphere.h
  \brief This class implements a sphere object for ray-tracing. 
*/

Sphere::Sphere (const Parameters& p) : Node(p) {}
/*!
  \brief Creates a sphere.
*/
Sphere::Sphere(const Parameters& p,const Vector& c, const double& rad) : Node(p) {
  this->c = c;
  this->rad=rad;
  this->p.setPosition(c);
}

Sphere::Sphere(const Sphere& s) : Node(s.p) {
  this->c = s.c;
  this->rad = s.rad;
}

/*!
  \brief Compute the intersection between the object and a ray
*/
int Sphere::Intersect(const Ray &ray,Intersections& l) const {

  double t=0.0;
  Vector op = c-ray.Origin();
  double b=op*ray.Direction();
  double det=b*b-op*op+rad*rad; 
  if (det<=0.0) return 0; 
  det=sqrt(det); 
 
  t=b-det;
  Vector pt = ray(t);
  l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(pt-c),pt, t));
 
  t=b+det;
  Vector pt2 = ray(t);
  l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(pt2-c),pt2, t));

  return 1;
 
} 

int Sphere::Inside (const Vector& pt ) const{
  if (  Norm(c-pt) - 1e-4 < rad )    return 1;
  return 0;
}

int Sphere::Inside (const Node&) const{
  return 1;
}

