/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "Texture.h"
#include <cmath>   

Texture::Texture(){
  Texture::color = Vector(0.0f,0.0f,0.0f);
}
Texture::Texture(const Texture& t){
  Texture::color = t.color;
}
Texture::Texture(const Vector& c){
  Texture::color = c;
}

/**
 * Texture par défaut
 */
TextureDefault::TextureDefault():Texture(){}
TextureDefault::TextureDefault(const Vector& c):Texture(c){}

TextureDefault::TextureDefault(const TextureDefault& t):Texture(t.color){}



Vector TextureDefault::getColor() const{
  return color;
}

Vector TextureDefault::getColorPt(const Vector& pt) const{
  return color;
}

double TextureDefault::getColor(int i) const{
  return color[i];
}

/**
 * Texture "aléatoire" (ne donne pas grand chose...)
 */
TextureRandom::TextureRandom(double min, double max):Texture(){
  TextureRandom::min = min;
  TextureRandom::max = max;
}

TextureRandom::TextureRandom(const TextureRandom& t) : Texture(t.color){  TextureRandom::min = t.min;
  TextureRandom::max = t.max;
}


Vector TextureRandom::getColor() const{
  return Vector(
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0,
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0,
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0);
}

Vector TextureRandom::getColorPt(const Vector& pt) const{
  return Vector(
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0,
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0,
		(rand()%(int)(max*10000.0)+min*10000.0)/9999.0);
}


/**
 * Texture style bois
 */
double TextureRandom::getColor(int i) const{
  return (rand()%(int)(max*10000.0)+min*10000.0)/9999.0;
}

TextureFct::TextureFct(const Vector& color1, const Vector& color2):Texture(){
  TextureFct::color1 = color1;
  TextureFct::color2 = color2;
}

TextureFct::TextureFct(const TextureFct& t) : Texture(t.color){
  TextureFct::color1 = t.color1;
  TextureFct::color2 = t.color2;
}


Vector TextureFct::getColor() const{
  return (color2+color1)/2;
}

Vector TextureFct::getColorPt(const Vector& pt) const{
  double d = sqrt(pt[0]*pt[0] + pt[1]*pt[1]);
  double u = (1+cos(d))/2;
  Vector result = (1-u) * color1 + u * color2;
  return result;
}


double TextureFct::getColor(int i) const{
  return color2[i]>color1[i]?color2[i]:color1[i];
}


/**
 * Texture en damier
 */

TextureDamier::TextureDamier(const Vector& color1, const Vector& color2, int step):Texture(){
  TextureDamier::step = step;
  TextureDamier::color1 = color1;
  TextureDamier::color2 = color2;
}

TextureDamier::TextureDamier(const TextureDamier& t) : Texture(t.color){  TextureDamier::step = t.step;
  TextureDamier::color1 = t.color1;
  TextureDamier::color2 = t.color2;
}




Vector TextureDamier::getColor() const{
  return (color2+color1)/2;
}

Vector TextureDamier::getColorPt(const Vector& pt) const{
  return (((int)pt[0]/step  + (int)pt[1]/step  + (int)pt[2]/step )%2==0) ? color1 : color2;
}


double TextureDamier::getColor(int i) const{
  return color2[i]>color1[i]?color2[i]:color1[i];
}

