/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "Tore.h"

Tore::Tore (const Parameters& p, const Vector& centre, const Vector& axe , double r1, double r2) : Node(p) {
  Tore::centre = centre;
  Tore::axeZ = axe;
  Tore::r1 = r1;
  Tore::r2 = r2;

  if(axeZ[0] > 0.5 || axeZ[0] < -0.5 || axeZ[1] > 0.5 || axeZ[1] < -0.5) {
    axeY[0] = axeZ[1];
    axeY[1] = -axeZ[0];
    axeY[2] = 0.0;
  } else {
    axeY[0] = 0.0;
    axeY[1] = axeZ[2];
    axeY[2] = -axeZ[1];
  }
  Normalized(axeY);
  Tore::axeX = Normalized(axeZ / axeY);
}


Tore::Tore (const Tore& t ) : Node(t.p){
  Tore::centre = t.centre;
  Tore::axeZ = t.axeZ;
  Tore::axeX = t.axeX;
  Tore::axeY = t.axeY;
  Tore::r1 = t.r1;
  Tore::r2 = t.r2;
}




int Tore::Inside (const Vector& p ) const{
  return 1;
}

int Tore::Inside (const Node&) const{
  return 1;
}

//(x² + y² + z² + R² - r²)² - 4 R² (x² + z²) = 0
int Tore::Intersect(const Ray& ray, Intersections& l) const{
  double A = 1.0;
  Vector viewPosRel = ray.Origin() - centre; 
  double udotp = (ray.Direction() * viewPosRel);
  double B = (udotp + udotp + udotp + udotp);
  double RadiiSqSum = r1*r1 + r2*r2;
  double ucdotp = (axeZ * viewPosRel);
  double ucdotu = (axeZ * ray.Direction());
  double pSq = (viewPosRel * viewPosRel);
  double C = B * udotp + (pSq + pSq) - (RadiiSqSum + RadiiSqSum) + 4.0 * (r1*r1) * ucdotu * ucdotu;
  double D = 4.0 * ((pSq - RadiiSqSum) * udotp + ((r1*r1) + (r1*r1)) * ucdotp * ucdotu);
  double E = (pSq - (RadiiSqSum + RadiiSqSum)) * pSq + 4.0 * (r1*r1) * ucdotp * ucdotp
    + (((r1*r1) - (r2*r2)) * ((r1*r1) - (r2*r2)));
  Polynomial poly (A, B, C, D, E);
  double roots[4] = { 0.0, 0.0, 0.0, 0.0 };
  int nbRoots = poly.SolveQuartic (roots);

  int result = 0;
  if (nbRoots > 0){    
    for (int ii = 0; ii < nbRoots; ii++){
      if (roots[ii] > 1e-4){

	Vector ptI = roots[ii] * ray.Direction() + ray.Origin();
	Vector ptTmp = ptI - centre;
	Vector pt;
	pt[0] = ptTmp * axeX;
	pt[1] = ptTmp * axeY;
	pt[2] = ptTmp * axeZ;
	Vector normal = Normalized(axeZ * (- pt[2]) + ptTmp);
	normal = Normalized(normal * (-r1) + ptTmp);
	l.append(Parameters(p.getEmission(),p.getTexture(),p.getReflection(),Normalized(normal),pt, roots[ii]));
	result=1;
      }
    }
  }
  return result;
}
